%% Reproduce results of pFISTA-parallel in the follwing paper:
%% Xinlin Zhang  et al., A Guaranteed Convergence Analysis for the Projected Fast Iterative Soft-Thresholding Algorithm in Parallel MRI, Medical Image Analysis, 69: 101987, 2021.
%% Created by Xinlin Zhang
%% Corresponding Author: Xiaobo Qu
%% Homepage: http://csrc.xmu.edu.cn
%% Email: quxiaobo <|at|> xmu.edu.cn
%% Affiliations: Computational Sensing Group (CSG), Departments of Electronic Science, Xiamen University, Xiamen 361005, China
%% All rights are reserved by CSG.
%% 2021-02
%%
%% Note: The codes were tested on Windows 10 (MATLAB 2019b) and Linux (MATLAB 2017b) platforms.

%%
clc
clear all
close all
%%
currentFolder = pwd;
addpath(genpath(currentFolder));

%% Load data
load data_slice10_4coil_cut
load mask_GC_34

%%
im_ori_sos = sos(im_ori);
im_ori = im_ori ./ max(im_ori_sos(:));
im_ref = im_ori_sos ./ max(im_ori_sos(:));

[row, col, coil] = size(im_ori);

k_ori = fft2c(im_ori);
k_sampled = mask .* k_ori;

%% Select method
method = 'SPIRiT'; % 'SENSE' or 'SPIRiT'

switch method
    case{'SENSE'}
        % Load sensitivity
        load sensitivity_4coil
        
        % Normalize sensitivity satissfing C^H*C = I
        sensitivity = sensitivity ./ sos(sensitivity);
        
        % Generate system matrix
        Fu = Fu_SENSE(mask, sensitivity);
        
        % Calculate 1/gamma
        c = 1;
        
    case{'SPIRiT'}
        % Setting parameter for SPIRiT
        kSize = [5,5];
        CalibTyk = 0.01;
        lambda_1 = 1e0;
        
        % Calculate SPIRiT kernel
        [CalibSize, dcomp] = getCalibSize(mask);
        kCalib = crop(k_sampled,[CalibSize,coil]);
        kernel = zeros([kSize,coil,coil]);
        [AtA] = corrMatrix(kCalib,kSize);
        for n=1:coil
            kernel(:,:,:,n) = calibrate(AtA,kSize,coil,n,CalibTyk);
        end
        GOP = SPIRiT(kernel, 'image',[row,col]);
        
        % Generate system matrix
        Fu = Fu_SPIRiT(mask, GOP, lambda_1);
        
        % Calculate 1/gamma
        weights = kernel2weights(kernel, [row,col]);
        c = calc_c_SPIRiT(weights, lambda_1);
        
        k_sampled = [k_sampled; zeros(size(k_sampled))];
end

%% Tight frame SIDWT paramters
level = 4;
wName = 'db4';
psi = UWT(level, wName);

%% Recon by pFISTA-parallel
params.psi       = psi;
params.Fu        = Fu;
params.lambda    = 1e-4;
params.tol_error = 1e-4;
params.iter_max  = 400;
params.im_ref    = im_ref;
params.gamma     = 1/c;

out = Solver_pFISTA_parallel(k_sampled, params);

%% Display results
im_rec = out.im_rec;

im_rec_show = sos(im_rec);
im_rec_show = im_rec_show ./ max(im_rec_show(:));

rlne_pfista = rlne(im_ref, im_rec_show)

figure;
subplot(2,2,1);imshow(im_ref);title('Reference')
subplot(2,2,2);imshow(im_rec_show);title(['Reconstructed by pFISTA-',method])
subplot(2,2,3);imshow(mask);title('Sampling pattern')
subplot(2,2,4);imshow(abs(im_rec_show-im_ref)*10);title('Error (10x)')